@@ -212,12 +212,17 @@ class _DataTypeDef(object):
 		return not self.is_scalar
 
 _DATA_TYPE_UNDEFINED = _DataTypeDef('UNDEFINED', errors.UNDEFINED)
-class _ArrayDataTypeDef(_DataTypeDef):
+class _CollectionDataTypeDef(_DataTypeDef):
 	__slots__ = ('value_type', 'value_type_nullable')
 	def __init__(self, name, python_type, value_type=_DATA_TYPE_UNDEFINED, value_type_nullable=True):
-		if not issubclass(python_type, collections.abc.Sequence):
-			raise TypeError('the specified python_type is not an array')
-		super(_ArrayDataTypeDef, self).__init__(name, python_type)
+		# check these three classes individually instead of using Collection which isn't available before Python v3.6
+		if not issubclass(python_type, collections.abc.Container):
+			raise TypeError('the specified python_type is not a container')
+		if not issubclass(python_type, collections.abc.Iterable):
+			raise TypeError('the specified python_type is not an iterable')
+		if not issubclass(python_type, collections.abc.Sized):
+			raise TypeError('the specified python_type is not a sized')
+		super(_CollectionDataTypeDef, self).__init__(name, python_type)
 		self.is_scalar = False
 		self.value_type = value_type
 		self.value_type_nullable = value_type_nullable
@@ -249,6 +254,12 @@ class _ArrayDataTypeDef(_DataTypeDef):
 	def __hash__(self):
 		return hash((self.python_type, self.is_scalar, hash((self.value_type, self.value_type_nullable))))
 
+class _ArrayDataTypeDef(_CollectionDataTypeDef):
+	pass
+
+class _SetDataTypeDef(_CollectionDataTypeDef):
+	pass
+
 class _MappingDataTypeDef(_DataTypeDef):
 	__slots__ = ('key_type', 'value_type', 'value_type_nullable')
 	def __init__(self, name, python_type, key_type=_DATA_TYPE_UNDEFINED, value_type=_DATA_TYPE_UNDEFINED, value_type_nullable=True):
@@ -256,6 +267,8 @@ class _MappingDataTypeDef(_DataTypeDef):
 			raise TypeError('the specified python_type is not a mapping')
 		super(_MappingDataTypeDef, self).__init__(name, python_type)
 		self.is_scalar = False
+		# ARRAY is the only compound data type that can be used as a mapping key, this is because ARRAY's are backed by
+		# Python tuple's while SET and MAPPING objects are set and dict instances, respectively which are not hashable.
 		if key_type.is_compound and not isinstance(key_type, DataType.ARRAY.__class__):
 			raise errors.EngineError("the {} data type may not be used for mapping keys".format(key_type.name))
 		self.key_type = key_type
@@ -316,7 +329,29 @@ class DataTypeMeta(type):
 
 class DataType(metaclass=DataTypeMeta):
 	"""
-	A collection of constants representing the different supported data types.
+	A collection of constants representing the different supported data types. There are three ways to compare data
+	types. All three are effectively the same when dealing with scalars.
+
+	Equality checking
+	  .. code-block::
+
+	    dt == DataType.TYPE
+	  This is the most explicit form of testing and when dealing with compound data types, it recursively checks that
+	  all of the member types are also equal.
+
+	Class checking
+	  .. code-block::
+
+	    isinstance(dt, DataType.TYPE.__class__)
+	  This checks that the data types are the same but when dealing with compound data types, the member types are
+	  ignored.
+
+	Compatibility checking
+	  .. code-block::
+
+	    DataType.is_compatible(dt, DataType.TYPE)
+	  This checks that the types are compatible without any kind of conversion. When dealing with compound data types,
+	  this ensures that the member types are either the same or :py:attr:`~.UNDEFINED`.
 	"""
 	ARRAY = _ArrayDataTypeDef('ARRAY', tuple)
 	"""
@@ -337,6 +372,13 @@ class DataType(metaclass=DataTypeMeta):
 	:param bool value_type_nullable: Whether or not mapping values are allowed to be :py:attr:`.NULL`.
 	"""
 	NULL = _DataTypeDef('NULL', NoneType)
+	SET = _SetDataTypeDef('SET', set)
+	"""
+	.. py:function:: __call__(value_type, value_type_nullable=True)
+
+	:param value_type: The type of the set members.
+	:param bool value_type_nullable: Whether or not set members are allowed to be :py:attr:`.NULL`.
+	"""
 	STRING = _DataTypeDef('STRING', str)
 	UNDEFINED = _DATA_TYPE_UNDEFINED
 	"""
@@ -384,6 +426,8 @@ class DataType(metaclass=DataTypeMeta):
 			return cls.MAPPING
 		elif python_type is NoneType:
 			return cls.NULL
+		elif python_type is set:
+			return cls.SET
 		elif python_type is str:
 			return cls.STRING
 		raise ValueError("can not map python type {0!r} to a compatible data type".format(python_type.__name__))
@@ -406,6 +450,8 @@ class DataType(metaclass=DataTypeMeta):
 			return cls.FLOAT
 		elif python_value is None:
 			return cls.NULL
+		elif isinstance(python_value, (set,)):
+			return cls.SET(value_type=_iterable_member_value_type(python_value))
 		elif isinstance(python_value, (str,)):
 			return cls.STRING
 		elif isinstance(python_value, collections.abc.Mapping):
@@ -438,14 +484,16 @@ class DataType(metaclass=DataTypeMeta):
 		if dt1.is_scalar and dt2.is_scalar:
 			return dt1 == dt2
 		elif dt1.is_compound and dt2.is_compound:
-			if isinstance(dt1, _ArrayDataTypeDef) and isinstance(dt2, _ArrayDataTypeDef):
+			if isinstance(dt1, DataType.ARRAY.__class__) and isinstance(dt2, DataType.ARRAY.__class__):
 				return cls.is_compatible(dt1.value_type, dt2.value_type)
-			elif isinstance(dt1, _MappingDataTypeDef) and isinstance(dt2, _MappingDataTypeDef):
+			elif isinstance(dt1, DataType.MAPPING.__class__) and isinstance(dt2, DataType.MAPPING.__class__):
 				if not cls.is_compatible(dt1.key_type, dt2.key_type):
 					return False
 				if not cls.is_compatible(dt1.value_type, dt2.value_type):
 					return False
 				return True
+			elif isinstance(dt1, DataType.SET.__class__) and isinstance(dt2, DataType.SET.__class__):
+				return cls.is_compatible(dt1.value_type, dt2.value_type)
 		return False
 
 	@classmethod
@@ -526,13 +574,15 @@ class LiteralExpressionBase(ExpressionBase):
 		"""
 		datatype = DataType.from_value(value)
 		for subclass in cls.__subclasses__():
+			if subclass.__name__.startswith('_'):
+				continue
 			if DataType.is_compatible(subclass.result_type, datatype):
 				break
 		else:
 			raise errors.EngineError("can not create literal expression from python value: {!r}".format(value))
 		if datatype.is_compound:
-			if isinstance(datatype, DataType.ARRAY.__class__):
-				value = tuple(cls.from_value(context, v) for v in value)
+			if isinstance(datatype, _CollectionDataTypeDef):
+				value = datatype.python_type(cls.from_value(context, v) for v in value)
 			elif isinstance(datatype, DataType.MAPPING.__class__):
 				value = tuple((cls.from_value(context, k), cls.from_value(context, v)) for k, v in value.items())
 		else:
@@ -551,26 +601,27 @@ class LiteralExpressionBase(ExpressionBase):
 ################################################################################
 # Literal Expressions
 ################################################################################
-class ArrayExpression(LiteralExpressionBase):
-	"""Literal array expressions containing 0 or more sub-expressions."""
-	result_type = DataType.ARRAY
-	def __init__(self, *args, **kwargs):
-		super(ArrayExpression, self).__init__(*args, **kwargs)
-		self.result_type = DataType.ARRAY(value_type=_iterable_member_value_type(self.value))
-
+class _CollectionMixin(object):
 	def evaluate(self, thing):
-		return tuple(member.evaluate(thing) for member in self.value)
+		return self.result_type.python_type(member.evaluate(thing) for member in self.value)
 
 	@property
 	def is_reduced(self):
 		return _is_reduced(*self.value)
 
 	def to_graphviz(self, digraph, *args, **kwargs):
-		super(ArrayExpression, self).to_graphviz(digraph, *args, **kwargs)
+		super(_CollectionMixin, self).to_graphviz(digraph, *args, **kwargs)
 		for member in self.value:
 			member.to_graphviz(digraph, *args, **kwargs)
 			digraph.edge(str(id(self)), str(id(member)))
 
+class ArrayExpression(_CollectionMixin, LiteralExpressionBase):
+	"""Literal array expressions containing 0 or more sub-expressions."""
+	result_type = DataType.ARRAY
+	def __init__(self, *args, **kwargs):
+		super(ArrayExpression, self).__init__(*args, **kwargs)
+		self.result_type = DataType.ARRAY(value_type=_iterable_member_value_type(self.value))
+
 class BooleanExpression(LiteralExpressionBase):
 	"""Literal boolean expressions representing True or False."""
 	result_type = DataType.BOOLEAN
@@ -635,6 +686,13 @@ class NullExpression(LiteralExpressionBase):
 			raise TypeError('value must be None')
 		super(NullExpression, self).__init__(context, value=None)
 
+class SetExpression(_CollectionMixin, LiteralExpressionBase):
+	"""Literal set expressions containing 0 or more sub-expressions."""
+	result_type = DataType.SET
+	def __init__(self, *args, **kwargs):
+		super(SetExpression, self).__init__(*args, **kwargs)
+		self.result_type = DataType.SET(value_type=_iterable_member_value_type(self.value))
+
 class StringExpression(LiteralExpressionBase):
 	"""Literal string expressions representing an array of characters."""
 	result_type = DataType.STRING
@@ -647,13 +705,12 @@ class LeftOperatorRightExpressionBase(ExpressionBase):
 	A base class for representing complex expressions composed of a left side and a right side, separated by an
 	operator.
 	"""
-	compatible_types = (DataType.ARRAY, DataType.BOOLEAN, DataType.DATETIME, DataType.FLOAT, DataType.MAPPING, DataType.NULL, DataType.STRING)
+	compatible_types = (DataType.ARRAY, DataType.BOOLEAN, DataType.DATETIME, DataType.FLOAT, DataType.MAPPING, DataType.NULL, DataType.SET, DataType.STRING)
 	"""
 	A tuple containing the compatible data types that the left and right expressions must return. This can for example
 	be used to indicate that arithmetic operations are compatible with :py:attr:`~.DataType.FLOAT` but not
 	:py:attr:`~.DataType.STRING` values.
 	"""
-	result_expression = BooleanExpression
 	result_type = DataType.BOOLEAN
 	def __init__(self, context, type_, left, right):
 		"""
@@ -693,7 +750,7 @@ class LeftOperatorRightExpressionBase(ExpressionBase):
 	def reduce(self):
 		if not _is_reduced(self.left, self.right):
 			return self
-		return self.result_expression(self.context, self.evaluate(None))
+		return LiteralExpressionBase.from_value(self.context, self.evaluate(None))
 
 	def to_graphviz(self, digraph, *args, **kwargs):
 		digraph.node(str(id(self)), "{}\ntype={!r}".format(self.__class__.__name__, self.type))
@@ -705,7 +762,6 @@ class LeftOperatorRightExpressionBase(ExpressionBase):
 class ArithmeticExpression(LeftOperatorRightExpressionBase):
 	"""A class for representing arithmetic expressions from the grammar text such as addition and subtraction."""
 	compatible_types = (DataType.FLOAT,)
-	result_expression = FloatExpression
 	result_type = DataType.FLOAT
 	def __op_arithmetic(self, op, thing):
 		left_value = self.left.evaluate(thing)
@@ -726,28 +782,56 @@ class BitwiseExpression(LeftOperatorRightExpressionBase):
 	"""
 	A class for representing bitwise arithmetic expressions from the grammar text such as XOR and shifting operations.
 	"""
-	compatible_types = (DataType.FLOAT,)
-	result_expression = FloatExpression
-	result_type = DataType.FLOAT
+	compatible_types = (DataType.FLOAT, DataType.SET)
+	result_type = DataType.UNDEFINED
 	def __init__(self, *args, **kwargs):
 		super(BitwiseExpression, self).__init__(*args, **kwargs)
-		if _is_reduced(self.left):
-			_assert_is_natural_number(self.left.evaluate(None))
-		if _is_reduced(self.right):
-			_assert_is_natural_number(self.right.evaluate(None))
+		# don't use DataType.is_compatible, because for sets the member type isn't important
+		if self.left.result_type != DataType.UNDEFINED and self.right.result_type != DataType.UNDEFINED:
+			if self.left.result_type.__class__ != self.right.result_type.__class__:
+				raise errors.EvaluationError('data type mismatch')
+		if self.left.result_type == DataType.FLOAT:
+			if _is_reduced(self.left):
+				_assert_is_natural_number(self.left.evaluate(None))
+			self.result_type = DataType.FLOAT
+		if self.right.result_type == DataType.FLOAT:
+			if _is_reduced(self.right):
+				_assert_is_natural_number(self.right.evaluate(None))
+			self.result_type = DataType.FLOAT
+		if isinstance(self.left.result_type, DataType.SET.__class__) or isinstance(self.right.result_type, DataType.SET.__class__):
+			self.result_type = DataType.SET  # this discards the member type info
 
-	def __op_bitwise(self, op, thing):
+	def _op_bitwise(self, op, thing):
 		left = self.left.evaluate(thing)
+		if DataType.from_value(left) == DataType.FLOAT:
+			return self._op_bitwise_float(op, thing, left)
+		elif isinstance(DataType.from_value(left), DataType.SET.__class__):
+			return self._op_bitwise_set(op, thing, left)
+		raise errors.EvaluationError('data type mismatch')
+
+	def _op_bitwise_float(self, op, thing, left):
 		_assert_is_natural_number(left)
 		right = self.right.evaluate(thing)
 		_assert_is_natural_number(right)
 		return _to_decimal(op(int(left), int(right)))
 
-	_op_bwand = functools.partialmethod(__op_bitwise, operator.and_)
-	_op_bwor  = functools.partialmethod(__op_bitwise, operator.or_)
-	_op_bwxor = functools.partialmethod(__op_bitwise, operator.xor)
-	_op_bwlsh = functools.partialmethod(__op_bitwise, operator.lshift)
-	_op_bwrsh = functools.partialmethod(__op_bitwise, operator.rshift)
+	def _op_bitwise_set(self, op, thing, left):
+		right = self.right.evaluate(thing)
+		if not DataType.is_compatible(DataType.from_value(right), DataType.SET):
+			raise errors.EvaluationError('data type mismatch')
+		return op(left, right)
+
+	_op_bwand = functools.partialmethod(_op_bitwise, operator.and_)
+	_op_bwor  = functools.partialmethod(_op_bitwise, operator.or_)
+	_op_bwxor = functools.partialmethod(_op_bitwise, operator.xor)
+
+class BitwiseShiftExpression(BitwiseExpression):
+	compatible_types = (DataType.FLOAT,)
+	result_type = DataType.FLOAT
+	def _op_bitwise_shift(self, *args, **kwargs):
+		return self._op_bitwise(*args, **kwargs)
+	_op_bwlsh = functools.partialmethod(_op_bitwise_shift, operator.lshift)
+	_op_bwrsh = functools.partialmethod(_op_bitwise_shift, operator.rshift)
 
 class LogicExpression(LeftOperatorRightExpressionBase):
 	"""A class for representing logical expressions from the grammar text such as "and" and "or"."""
@@ -763,9 +847,8 @@ class LogicExpression(LeftOperatorRightExpressionBase):
 class ComparisonExpression(LeftOperatorRightExpressionBase):
 	"""A class for representing comparison expressions from the grammar text such as equality checks."""
 	def _op_eq(self, thing):
-		if self.left.result_type != DataType.UNDEFINED and self.right.result_type != DataType.UNDEFINED:
-			if not DataType.is_compatible(self.left.result_type, self.right.result_type):
-				return False
+		if not DataType.is_compatible(self.left.result_type, self.right.result_type):
+			return False
 		left_value = self.left.evaluate(thing)
 		right_value = self.right.evaluate(thing)
 		if type(left_value) is not type(right_value):
@@ -773,9 +856,8 @@ class ComparisonExpression(LeftOperatorRightExpressionBase):
 		return operator.eq(left_value, right_value)
 
 	def _op_ne(self, thing):
-		if self.left.result_type != DataType.UNDEFINED and self.right.result_type != DataType.UNDEFINED:
-			if not DataType.is_compatible(self.left.result_type, self.right.result_type):
-				return True
+		if not DataType.is_compatible(self.left.result_type, self.right.result_type):
+			return True
 		left_value = self.left.evaluate(thing)
 		right_value = self.right.evaluate(thing)
 		if type(left_value) is not type(right_value):
@@ -990,6 +1072,8 @@ class GetItemExpression(ExpressionBase):
 			if not (safe or DataType.is_compatible(item.result_type, container.result_type.key_type)):
 				raise errors.LookupError(errors.UNDEFINED, errors.UNDEFINED)
 			self.result_type = container.result_type.value_type
+		elif isinstance(container.result_type, DataType.SET.__class__):
+			raise errors.EvaluationError('data type mismatch (container is a set)')
 		elif container.result_type != DataType.UNDEFINED:
 			if not (container.result_type == DataType.NULL and safe):
 				raise errors.EvaluationError('data type mismatch')
@@ -1056,6 +1140,8 @@ class GetSliceExpression(ExpressionBase):
 		# is explicitly checking if result_type is an array with out checking the value_type
 		elif isinstance(container.result_type, DataType.ARRAY.__class__):
 			self.result_type = container.result_type
+		elif isinstance(container.result_type, DataType.SET.__class__):
+			raise errors.EvaluationError('data type mismatch (container is a set)')
 		elif container.result_type != DataType.UNDEFINED:
 			if not (container.result_type == DataType.NULL and safe):
 				raise errors.EvaluationError('data type mismatch')
