@@ -288,13 +288,20 @@ class Parser(ParserBase):
 		expression : expression BWAND  expression
 				   | expression BWOR   expression
 				   | expression BWXOR  expression
-				   | expression BWLSH  expression
-				   | expression BWRSH  expression
 		"""
 		left, op, right = p[1:4]
 		op_name = self.op_names[op]
 		p[0] = ast.BitwiseExpression(self.context, op_name, left, right).reduce()
 
+	def p_expression_bitwise_shift(self, p):
+		"""
+		expression : expression BWLSH  expression
+				   | expression BWRSH  expression
+		"""
+		left, op, right = p[1:4]
+		op_name = self.op_names[op]
+		p[0] = ast.BitwiseShiftExpression(self.context, op_name, left, right).reduce()
+
 	def p_expression_contains(self, p):
 		"""
 		expression : expression IN     expression
@@ -399,6 +406,13 @@ class Parser(ParserBase):
 		# null is an object because of the safe operator
 		p[0] = ast.NullExpression(self.context)
 
+	def p_expression_set(self, p):
+		"""
+		object : LBRACE ary_members RBRACE
+			   | LBRACE ary_members COMMA RBRACE
+		"""
+		p[0] = ast.SetExpression(self.context, tuple(p[2])).reduce()
+
 	def p_expression_string(self, p):
 		'object : STRING'
 		p[0] = ast.StringExpression(self.context, literal_eval(p[1]))
